/*
	Titel:		LCD-Test - 2x16 LCD Anzeige 4bit Mode
	Author:		Thorsten
	Datum:		03.10.2007
	Version:	1.0
	
	Update:		2.0 - 16.10.2007	+set_cursor & LCD_data & LCD_command
									+LCDhome & LCDclear
									-LCDout
									Timing Verbesserung
	
	Software:	AVR-GCC 4.1.2
	Hardware:	ATMEL AVR
*/

#include "lcd.h"

void LCDenable(void)
{
	LCD_PORT |= (1<<en);
	_delay_us(0.25);
	LCD_PORT &= ~(1<<en);
	_delay_us(0.25);
}

void LCDinit(void)
{
	unsigned char j;
	LCD_DDR = 0x00;
	LCD_DDR |= ((1<<en) | (1<<rs));
	LCD_DDR |= 0xf0;
	_delay_ms(10);							//2 Delays to use with XTAL 20MHz !
	_delay_ms(5);
	for (j=0;j<3;j++)
	{
		LCD_PORT &= ~((1<<en) | (1<<rs));
		LCD_PORT = 0x20;					//init Display 4 bit / 2 Zeilen / 5x7 Dot
		LCDenable();
		LCD_PORT = 0x80;
		LCDenable();
		_delay_ms(4.1);
	}

	LCD_PORT = 0x00;						//Display / ein Cursor aus / kein blinken
	LCDenable();
	LCD_PORT = 0xC0;						
	LCDenable();
	_delay_ms(0.042);
	
	LCD_PORT = 0x00;						//inkrement keine schrollen
	LCDenable();
	LCD_PORT = 0x60;						
	LCDenable();
	_delay_ms(0.042);
	
	LCD_PORT = 0x00;						//clear Display
	LCDenable();
	LCD_PORT = 0x10;						
	LCDenable();
	_delay_ms(1.64);
	
}

void sendchr(char zei)
{
	unsigned char temp;
	
	temp = (zei & 0xf0);
	LCD_PORT &= 0x0F;
	LCD_PORT |= temp;
	LCDenable();
	
	temp = ((zei<<4) & 0xf0);
	LCD_PORT &= 0x0F;
	LCD_PORT |= temp;
	LCDenable();
	
	_delay_ms(0.042);
}

void LCD_command(unsigned char data)
{
	LCD_PORT &= ~(1<<rs);
	sendchr(data);
}

void LCD_data(unsigned char data)
{
	LCD_PORT |= (1<<rs);
	sendchr(data);
}

void LCDclear(void)
{
		LCD_command(CLEAR_DISPLAY);
		_delay_ms(1.64);
}		

void LCDhome(void)
{
		LCD_command(CURSOR_HOME);
		_delay_ms(1.64);
}		

void set_cursor(unsigned char x, unsigned char y)
{
	switch (y)
	{
		case 1: LCD_command(0x80+0x00+x); break;	//1.Zeile Display
		case 2: LCD_command(0x80+0x40+x); break;	//2.Zeile Display
	}
}	


void LCD(char* buffer)
{
	unsigned char outl, i;
	char zeichen;
	outl = strlen(buffer);
	for(i=0;i<outl;i++)
	{
		zeichen = buffer[i];
		LCD_data(zeichen);
	}
}

